"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""

from PyQt5.QtWidgets import QDialog
from .ui_dateintervaldialog import Ui_DateIntervalDialog
from datetime import date


class DateIntervalDialog(QDialog):
    def __init__(self, parent):
        QDialog.__init__(self)
        self._parent = parent
        self._address = None
        self._ui = Ui_DateIntervalDialog()
        self._ui.setupUi(self)
        self._ui.pbOk.setEnabled(True)
        self._ui.pbOk.clicked.connect(self.accept)

    def getInterval(self):
        cDate = self._ui.cwFrom.selectedDate()
        dateBeg = date(cDate.year(), cDate.month(), cDate.day())
        cDate = self._ui.cwTo.selectedDate()
        dateEnd = date(cDate.year(), cDate.month(), cDate.day())
        return dateBeg, dateEnd



"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""
import os
import locale
import sys
import json

from PyQt5.QtWidgets import QColorDialog, QWidget, QButtonGroup, QFileDialog
from PyQt5.QtGui import QColor, QIcon, QPixmap
from datetime import datetime
from .logprint import print
from .dateintervaldialog import DateIntervalDialog
from .afHasHead import HasHead
from .stats import Stats


class Prefs(QWidget):
    def __init__(self, parent, ui, settings):
        QWidget.__init__(self, parent)
        self._ui = ui
        self._parent = parent
        locale.setlocale(locale.LC_ALL, '')
        self._loadCbCountry()
        self._settings = settings
        self._plottingColors = dict()
        self._RMOBclient = "UNKNOWN"
        self._pressedButton = None
        self._tabStats = self._parent.tabStats
        self._dataSource = self._parent.dataSource
        self._sporadicDatesList = list()
        self._sporadicBackgroundByHour = ''
        self._sporadicBackgroundBy10min = ''
        self._attributeFiltersDict = dict()

        self._ssBase = "min-height: 20px; min-width: 20px; border: 1px solid yellow; background-color: {};"
        self._colorButtons = [self._ui.pbScolor, self._ui.pbNcolor, self._ui.pbDiffColor, self._ui.pbAvgDiffColor,
                              self._ui.pbUpperColor, self._ui.pbLowerColor, self._ui.pbCountsColor,
                              self._ui.pbGridsColor, self._ui.pbMinColor, self._ui.pbBackColor]
        self._colorKeys = ['S', 'N', 'diff', 'avgDiff', 'upperThr', 'lowerThr', 'counts', 'majorGrids', 'minorGrids',
                           'background']
        self._colorDict = self._settings.readSettingAsObject('colorDict')

        self._ui.cbFontFamily.currentTextChanged.connect(
            lambda text: self._settings.writeSetting('fontFamily', text))

        self._ui.cbFontStyle.currentTextChanged.connect(
            lambda text: self._settings.writeSetting('fontStyle', text))

        self._ui.cbDataLineStyle.currentTextChanged.connect(
            lambda text: self._settings.writeSetting('dataLineStyle', text))
        self._ui.cbMajorLineStyle.currentTextChanged.connect(
            lambda text: self._settings.writeSetting('majorLineStyle', text))
        self._ui.cbMinorLineStyle.currentTextChanged.connect(
            lambda text: self._settings.writeSetting('minorLineStyle', text))

        self._ui.sbFontSize.valueChanged.connect(lambda val: self._settings.writeSetting('fontSize', val))
        self._ui.sbDataLineWidth.valueChanged.connect(lambda val: self._settings.writeSetting('dataLineWidth', val))
        self._ui.sbMajorLineWidth.valueChanged.connect(lambda val: self._settings.writeSetting('majorLineWidth', val))
        self._ui.sbMinorLineWidth.valueChanged.connect(lambda val: self._settings.writeSetting('minorLineWidth', val))
        self._ui.chkTooltips.clicked.connect(lambda checked: self._settings.writeSetting('tooltipDisabled', checked))
        self._ui.chkAutosave.clicked.connect(lambda checked: self._settings.writeSetting('autosaving', checked))
        self._ui.chkShowCursor.clicked.connect(lambda checked: self._settings.writeSetting('cursorEnabled', checked))
        self._ui.cbDataSep.currentTextChanged.connect(lambda text: self._settings.writeSetting('separator', text))

        self._ui.leStation.textChanged.connect(lambda text: self._settings.writeSetting('stationName', text))
        self._ui.leOwner.textChanged.connect(lambda text: self._settings.writeSetting('owner', text))
        self._ui.cbCountry.currentTextChanged.connect(lambda text: self._settings.writeSetting('country', text))
        self._ui.leCity.textChanged.connect(lambda text: self._settings.writeSetting('city', text))
        self._ui.leLatitude.textChanged.connect(lambda text: self._settings.writeSetting('latitude', text))
        self._ui.leLongitude.textChanged.connect(lambda text: self._settings.writeSetting('longitude', text))
        self._ui.leLatitudeDeg.textChanged.connect(lambda text: self._settings.writeSetting('latitudeDeg', text))
        self._ui.leLongitudeDeg.textChanged.connect(lambda text: self._settings.writeSetting('longitudeDeg', text))
        self._ui.sbAltitude.valueChanged.connect(lambda val: self._settings.writeSetting('altitude', val))
        self._ui.pbLogo.pressed.connect(self._selectLogo)

        self._ui.leAntenna.textChanged.connect(lambda text: self._settings.writeSetting('antenna', text))
        self._ui.sbAntennaAzimuth.valueChanged.connect(lambda val: self._settings.writeSetting('antAzimuth', val))
        self._ui.sbAntennaElevation.valueChanged.connect(lambda val: self._settings.writeSetting('antElevation', val))
        self._ui.lePre.textChanged.connect(lambda text: self._settings.writeSetting('preamplifier', text))
        self._ui.leFrequencies.textChanged.connect(lambda text: self._settings.writeSetting('frequencies', text))
        self._ui.leReceiver.textChanged.connect(lambda text: self._settings.writeSetting('receiver', text))
        self._ui.leComputer.textChanged.connect(lambda text: self._settings.writeSetting('computer', text))
        self._ui.leEmail.textChanged.connect(lambda text: self._settings.writeSetting('email', text))
        self._ui.pteNotes.textChanged.connect(
            lambda: self._settings.writeSetting('notes', self._ui.pteNotes.toPlainText()))

        self._ui.chkESDfilter.clicked.connect(lambda checked: self._settings.writeSetting('ESDfilter', checked))
        self._ui.chkSatFilter.clicked.connect(lambda checked: self._settings.writeSetting('SATfilter', checked))
        self._ui.chkCarrierFilter1.clicked.connect(lambda checked: self._settings.writeSetting('CAR1filter', checked))
        self._ui.chkCarrierFilter2.clicked.connect(lambda checked: self._settings.writeSetting('CAR2filter', checked))
        self._ui.sbESDthreshold.valueChanged.connect(lambda val: self._settings.writeSetting('ESDfilterThreshold', val))
        self._ui.sbSATthreshold.valueChanged.connect(lambda val: self._settings.writeSetting('SATfilterThreshold', val))
        self._ui.sbCar1Threshold.valueChanged.connect(
            lambda val: self._settings.writeSetting('CAR1filterThreshold', val))
        self._ui.sbCar2Threshold.valueChanged.connect(
            lambda val: self._settings.writeSetting('CAR2filterThreshold', val))
        self._ui.sbUnderThr.valueChanged.connect(lambda val: self._settings.writeSetting('underdenseMs', val))
        self._ui.sbOverThr.valueChanged.connect(lambda val: self._settings.writeSetting('overdenseSec', val))
        self._ui.pbFindRMOBclient.pressed.connect(self._selectRMOBclient)

        self._ui.pbAdd.pressed.connect(self._addDateInterval)
        self._ui.pbDel.pressed.connect(self._removeDateInterval)
        self._ui.pbCalc.pressed.connect(self._calculateSporadicBackground)
        # self._ui.pbEditParms.pressed.connect(self._editAttributeParms)

        self.updateTabPrefs()

    def updateTabPrefs(self):
        enableSB = 0
        avgDailyStr = self._settings.readSettingAsObject('sporadicBackgroundDaily')
        if len(avgDailyStr) > 0:
            enableSB |= 1

        avgHourStr = self._settings.readSettingAsObject('sporadicBackgroundByHour')
        if len(avgHourStr) > 0:
            enableSB |= 2

        avg10minStr = self._settings.readSettingAsObject('sporadicBackgroundBy10min')
        if len(avg10minStr) > 0:
            enableSB |= 4

        self._ui.pbAdd.setEnabled(self._parent.dbOk)
        self._ui.pbDel.setEnabled(self._parent.dbOk)
        self._ui.pbCalc.setEnabled(self._parent.dbOk)
        self._ui.lbSBok.setVisible(enableSB == 7)

        group = QButtonGroup(self)
        group.buttonClicked.connect(self._openColorDialog)
        for i in range(0, len(self._colorKeys)):
            colorKey = self._colorKeys[i]
            colorName = self._colorDict[colorKey].name()
            ss = self._ssBase.format(colorName)
            button = self._colorButtons[i]
            button.setStyleSheet(ss)
            button.setWhatsThis(colorKey)
            group.addButton(button)

        self._ui.chkESDfilter.setChecked(self._settings.readSettingAsBool('ESDfilter'))
        self._ui.chkSatFilter.setChecked(self._settings.readSettingAsBool('SATfilter'))
        self._ui.chkCarrierFilter1.setChecked(self._settings.readSettingAsBool('CAR1filter'))
        self._ui.chkCarrierFilter2.setChecked(self._settings.readSettingAsBool('CAR2filter'))
        self._ui.sbESDthreshold.setValue(self._settings.readSettingAsFloat('ESDfilterThreshold'))
        self._ui.sbSATthreshold.setValue(self._settings.readSettingAsFloat('SATfilterThreshold'))
        self._ui.sbCar1Threshold.setValue(self._settings.readSettingAsFloat('CAR1filterThreshold'))
        self._ui.sbCar2Threshold.setValue(self._settings.readSettingAsFloat('CAR2filterThreshold'))
        self._ui.sbUnderThr.setValue(self._settings.readSettingAsInt('underdenseMs'))
        self._ui.sbOverThr.setValue(self._settings.readSettingAsInt('overdenseSec'))

        # attribute filters structure
        # key=python module name
        # value={[{filter fullpath: val},{filter description: val}, [{parametro: valore}+]]

        # read the parameters of the filters stored in settings, if present
        # self._attributeFiltersDict = self._settings.readSettingAsObject('attributeFilters')
        # storedFilters = self._attributeFiltersDict['filters'].keys()

        # scans the attribute filters directory attributes/ and gets all the attribute filter classes
        foundFilters = sorted([m.__name__ for m in sys.modules.values() if m.__name__.startswith('af')])

        if len(foundFilters) > 0:
            self._ui.cbAttribSettings.addItems(foundFilters)

        self._ui.cbFontFamily.setCurrentText(self._settings.readSettingAsString('fontFamily'))
        self._ui.sbFontSize.setValue(self._settings.readSettingAsInt('fontSize'))
        self._ui.cbFontStyle.setCurrentText(self._settings.readSettingAsString('fontStyle'))

        self._ui.cbDataLineStyle.setCurrentText(self._settings.readSettingAsString('dataLineStyle'))
        self._ui.cbMajorLineStyle.setCurrentText(self._settings.readSettingAsString('majorLineStyle'))
        self._ui.cbMinorLineStyle.setCurrentText(self._settings.readSettingAsString('minorLineStyle'))
        self._ui.sbDataLineWidth.setValue(self._settings.readSettingAsFloat('dataLineWidth'))
        self._ui.sbMajorLineWidth.setValue(self._settings.readSettingAsFloat('majorLineWidth'))
        self._ui.sbMinorLineWidth.setValue(self._settings.readSettingAsFloat('minorLineWidth'))

        self._ui.chkTooltips.setChecked(self._settings.readSettingAsBool('tooltipDisabled'))
        self._ui.chkAutosave.setChecked(self._settings.readSettingAsBool('autosaving'))
        self._ui.chkShowCursor.setChecked(self._settings.readSettingAsBool('cursorEnabled'))

        self._ui.cbDataSep.setCurrentText(self._settings.readSettingAsString('separator'))

        self._ui.leStation.setText(self._settings.readSettingAsString('stationName'))
        self._ui.leOwner.setText(self._settings.readSettingAsString('owner'))
        self._ui.cbCountry.setCurrentText(self._settings.readSettingAsString('country'))
        self._ui.leCity.setText(self._settings.readSettingAsString('city'))
        self._ui.leLatitude.setText(self._settings.readSettingAsString('latitude'))
        self._ui.leLongitude.setText(self._settings.readSettingAsString('longitude'))
        self._ui.leLatitudeDeg.setText(self._settings.readSettingAsString('latitudeDeg'))
        self._ui.leLongitudeDeg.setText(self._settings.readSettingAsString('longitudeDeg'))
        self._ui.sbAltitude.setValue(self._settings.readSettingAsInt('altitude'))
        logoIcon = QIcon(self._settings.readSettingAsString('logoPath'))
        self._ui.pbLogo.setText('')
        self._ui.pbLogo.setIcon(logoIcon)
        self._ui.pbLogo.setIconSize(self._ui.pbLogo.size())
        self._ui.leAntenna.setText(self._settings.readSettingAsString('antenna'))
        self._ui.sbAntennaAzimuth.setValue(self._settings.readSettingAsInt('antAzimuth'))
        self._ui.sbAntennaElevation.setValue(self._settings.readSettingAsInt('antElevation'))
        self._ui.lePre.setText(self._settings.readSettingAsString('preamplifier'))
        self._ui.leFrequencies.setText(self._settings.readSettingAsString('frequencies'))
        self._ui.leReceiver.setText(self._settings.readSettingAsString('receiver'))
        self._ui.leComputer.setText(self._settings.readSettingAsString('computer'))
        self._ui.leEmail.setText(self._settings.readSettingAsString('email'))
        self._ui.pteNotes.setPlainText(self._settings.readSettingAsString('notes'))
        self._RMOBclient = self._settings.readSettingAsString('RMOBclient')
        self._ui.lbRMOBclientPath.setText(self._RMOBclient)
        self._ui.lbRMOBclientPath.setToolTip(self._RMOBclient)

        sdList = self._settings.readSettingAsObject('sporadicDates')
        if len(sdList) > 0:
            self._sporadicDatesList  = json.loads(sdList)
        # only for a new calculation, no need to retrieve actual values from settings
        self._sporadicBackgroundByHour = None
        self._sporadicBackgroundBy10min = None

        self._ui.lwSporadic.clear()
        if self._sporadicDatesList is not None:
            for intervalStr in self._sporadicDatesList:
                self._addDateInterval(intervalStr)
        else:
            self._sporadicDatesList = list()
        self._dataSource = self._parent.dataSource
        self._ui.pbCalc.setEnabled(self._dataSource is not None)

    def _addDateInterval(self, intervalStr: str = None):
        if intervalStr is None:
            did = DateIntervalDialog(self._ui)
            did.exec()
            diTuple = did.getInterval()
            intervalStr = diTuple[0].isoformat() + " -> " + diTuple[1].isoformat()
            self._sporadicDatesList.append(intervalStr)
            print("Adding date interval:", intervalStr)
            sdList = json.dumps(self._sporadicDatesList)
            self._settings.writeSetting('sporadicDates', sdList)

        self._ui.lwSporadic.addItem(intervalStr)

    def _removeDateInterval(self):
        item = self._ui.lwSporadic.currentItem()
        if item:
            intervalStr = item.text()
            print("Removing date interval:", intervalStr)
            row = self._ui.lwSporadic.row(item)
            self._ui.lwSporadic.takeItem(row)
            self._sporadicDatesList.remove(intervalStr)
            sdList = json.dumps(self._sporadicDatesList)
            self._settings.writeSetting('sporadicDates', sdList)

    def _calculateSporadicBackground(self):
        done = self._tabStats.calculateSporadicBackground()
        if done:
            self.updateTabPrefs()

    def _openColorDialog(self, button):
        print("clicked button#", button)
        color = QColorDialog.getColor()
        if color.isValid():
            button.setStyleSheet(self._ssBase.format(color.name()))
            colorKey = button.whatsThis()
            self._colorDict[colorKey] = QColor(color.name())

    def _selectLogo(self):
        logoTuple = QFileDialog.getOpenFileName(
            self, "Open logo image", '*', "logo images (*.png *.jpeg *.jpg)", '')

        logoPath = logoTuple[0]
        if logoPath is None:
            return

        logo = QPixmap(logoPath)
        logoIcon = QIcon(logo)
        self._ui.pbLogo.setText('')
        self._ui.pbLogo.setIcon(logoIcon)
        self._ui.pbLogo.setIconSize(self._ui.pbLogo.size())
        self._settings.writeSetting('logoPath', logoPath)

    def _selectRMOBclient(self):
        if os.name == 'nt':
            clientTuple = QFileDialog.getOpenFileName(
                self, "Open RMOB FTP client executable file", '*', "executable files(*.exe *.EXE)", '')
        else:
            clientTuple = QFileDialog.getOpenFileName(
                self, "Open RMOB FTP client executable file", '*', '', '')

        self._RMOBclient = clientTuple[0]
        self._ui.lbRMOBclientPath.setText(self._RMOBclient)
        self._ui.lbRMOBclientPath.setToolTip(self._RMOBclient)
        self._settings.writeSetting('RMOBclient', self._RMOBclient)

    def _loadCbCountry(self):
        countryDict = {
            "AF": "Afghanistan",
            "AL": "Albania",
            "DZ": "Algeria",
            "AS": "American Samoa",
            "AD": "Andorra",
            "AO": "Angola",
            "AI": "Anguilla",
            "AQ": "Antarctica",
            "AG": "Antigua And Barbuda",
            "AR": "Argentina",
            "AM": "Armenia",
            "AW": "Aruba",
            "AU": "Australia",
            "AT": "Austria",
            "AZ": "Azerbaijan",
            "BS": "Bahamas",
            "BH": "Bahrain",
            "BD": "Bangladesh",
            "BB": "Barbados",
            "BY": "Belarus",
            "BE": "Belgium",
            "BZ": "Belize",
            "BJ": "Benin",
            "BM": "Bermuda",
            "BT": "Bhutan",
            "BO": "Bolivia",
            "BA": "Bosnia And Herzegovina",
            "BW": "Botswana",
            "BV": "Bouvet Island",
            "BR": "Brazil",
            "IO": "British Indian Ocean Territory",
            "BN": "Brunei Darussalam",
            "BG": "Bulgaria",
            "BF": "Burkina Faso",
            "BI": "Burundi",
            "KH": "Cambodia",
            "CM": "Cameroon",
            "CA": "Canada",
            "CV": "Cape Verde",
            "KY": "Cayman Islands",
            "CF": "Central African Republic",
            "TD": "Chad",
            "CL": "Chile",
            "CN": "China",
            "CX": "Christmas Island",
            "CC": "Cocos (keeling) Islands",
            "CO": "Colombia",
            "KM": "Comoros",
            "CG": "Congo",
            "CD": "Congo, The Democratic Republic Of The",
            "CK": "Cook Islands",
            "CR": "Costa Rica",
            "CI": "Cote D'ivoire",
            "HR": "Croatia",
            "CU": "Cuba",
            "CY": "Cyprus",
            "CZ": "Czech Republic",
            "DK": "Denmark",
            "DJ": "Djibouti",
            "DM": "Dominica",
            "DO": "Dominican Republic",
            "TP": "East Timor",
            "EC": "Ecuador",
            "EG": "Egypt",
            "SV": "El Salvador",
            "GQ": "Equatorial Guinea",
            "ER": "Eritrea",
            "EE": "Estonia",
            "ET": "Ethiopia",
            "FK": "Falkland Islands (malvinas)",
            "FO": "Faroe Islands",
            "FJ": "Fiji",
            "FI": "Finland",
            "FR": "France",
            "GF": "French Guiana",
            "PF": "French Polynesia",
            "TF": "French Southern Territories",
            "GA": "Gabon",
            "GM": "Gambia",
            "GE": "Georgia",
            "DE": "Germany",
            "GH": "Ghana",
            "GI": "Gibraltar",
            "GR": "Greece",
            "GL": "Greenland",
            "GD": "Grenada",
            "GP": "Guadeloupe",
            "GU": "Guam",
            "GT": "Guatemala",
            "GN": "Guinea",
            "GW": "Guinea-bissau",
            "GY": "Guyana",
            "HT": "Haiti",
            "HM": "Heard Island And Mcdonald Islands",
            "VA": "Holy See (vatican City State)",
            "HN": "Honduras",
            "HK": "Hong Kong",
            "HU": "Hungary",
            "IS": "Iceland",
            "IN": "India",
            "ID": "Indonesia",
            "IR": "Iran, Islamic Republic Of",
            "IQ": "Iraq",
            "IE": "Ireland",
            "IL": "Israel",
            "IT": "Italy",
            "JM": "Jamaica",
            "JP": "Japan",
            "JO": "Jordan",
            "KZ": "Kazakstan",
            "KE": "Kenya",
            "KI": "Kiribati",
            "KP": "Korea, Democratic People's Republic Of",
            "KR": "Korea, Republic Of",
            "KV": "Kosovo",
            "KW": "Kuwait",
            "KG": "Kyrgyzstan",
            "LA": "Lao People's Democratic Republic",
            "LV": "Latvia",
            "LB": "Lebanon",
            "LS": "Lesotho",
            "LR": "Liberia",
            "LY": "Libyan Arab Jamahiriya",
            "LI": "Liechtenstein",
            "LT": "Lithuania",
            "LU": "Luxembourg",
            "MO": "Macau",
            "MK": "Macedonia, The Former Yugoslav Republic Of",
            "MG": "Madagascar",
            "MW": "Malawi",
            "MY": "Malaysia",
            "MV": "Maldives",
            "ML": "Mali",
            "MT": "Malta",
            "MH": "Marshall Islands",
            "MQ": "Martinique",
            "MR": "Mauritania",
            "MU": "Mauritius",
            "YT": "Mayotte",
            "MX": "Mexico",
            "FM": "Micronesia, Federated States Of",
            "MD": "Moldova, Republic Of",
            "MC": "Monaco",
            "MN": "Mongolia",
            "MS": "Montserrat",
            "ME": "Montenegro",
            "MA": "Morocco",
            "MZ": "Mozambique",
            "MM": "Myanmar",
            "NA": "Namibia",
            "NR": "Nauru",
            "NP": "Nepal",
            "NL": "Netherlands",
            "AN": "Netherlands Antilles",
            "NC": "New Caledonia",
            "NZ": "New Zealand",
            "NI": "Nicaragua",
            "NE": "Niger",
            "NG": "Nigeria",
            "NU": "Niue",
            "NF": "Norfolk Island",
            "MP": "Northern Mariana Islands",
            "NO": "Norway",
            "OM": "Oman",
            "PK": "Pakistan",
            "PW": "Palau",
            "PS": "Palestinian Territory, Occupied",
            "PA": "Panama",
            "PG": "Papua New Guinea",
            "PY": "Paraguay",
            "PE": "Peru",
            "PH": "Philippines",
            "PN": "Pitcairn",
            "PL": "Poland",
            "PT": "Portugal",
            "PR": "Puerto Rico",
            "QA": "Qatar",
            "RE": "Reunion",
            "RO": "Romania",
            "RU": "Russian Federation",
            "RW": "Rwanda",
            "SH": "Saint Helena",
            "KN": "Saint Kitts And Nevis",
            "LC": "Saint Lucia",
            "PM": "Saint Pierre And Miquelon",
            "VC": "Saint Vincent And The Grenadines",
            "WS": "Samoa",
            "SM": "San Marino",
            "ST": "Sao Tome And Principe",
            "SA": "Saudi Arabia",
            "SN": "Senegal",
            "RS": "Serbia",
            "SC": "Seychelles",
            "SL": "Sierra Leone",
            "SG": "Singapore",
            "SK": "Slovakia",
            "SI": "Slovenia",
            "SB": "Solomon Islands",
            "SO": "Somalia",
            "ZA": "South Africa",
            "GS": "South Georgia And The South Sandwich Islands",
            "ES": "Spain",
            "LK": "Sri Lanka",
            "SD": "Sudan",
            "SR": "Suriname",
            "SJ": "Svalbard And Jan Mayen",
            "SZ": "Swaziland",
            "SE": "Sweden",
            "CH": "Switzerland",
            "SY": "Syrian Arab Republic",
            "TW": "Taiwan, Province Of China",
            "TJ": "Tajikistan",
            "TZ": "Tanzania, United Republic Of",
            "TH": "Thailand",
            "TG": "Togo",
            "TK": "Tokelau",
            "TO": "Tonga",
            "TT": "Trinidad And Tobago",
            "TN": "Tunisia",
            "TR": "Turkey",
            "TM": "Turkmenistan",
            "TC": "Turks And Caicos Islands",
            "TV": "Tuvalu",
            "UG": "Uganda",
            "UA": "Ukraine",
            "AE": "United Arab Emirates",
            "GB": "United Kingdom",
            "US": "United States",
            "UM": "United States Minor Outlying Islands",
            "UY": "Uruguay",
            "UZ": "Uzbekistan",
            "VU": "Vanuatu",
            "VE": "Venezuela",
            "VN": "Viet Nam",
            "VG": "Virgin Islands, British",
            "VI": "Virgin Islands, U.s.",
            "WF": "Wallis And Futuna",
            "EH": "Western Sahara",
            "YE": "Yemen",
            "ZM": "Zambia",
            "ZW": "Zimbabwe"
        }
        self._ui.cbCountry.insertItems(0, countryDict.values())



"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""
from PyQt5.QtWidgets import QHBoxLayout, QScrollArea, QLabel
from .logprint import print

class Plots:

    PLTW_THUMBNAILS = 0
    PLTW_POWER = 1
    PLTW_CMAP = 2
    PLTW_3D = 3
    PLTW_DETAILS = 4

    def __init__(self, parent, ui):
        self._ui = ui
        self._parent = parent
        self._toDailyNr = 0
        self._currentDailyNr = 0
        self._plot = None
        self._dailyNr = 0
        self._utcDate = None
        self._ui.sbID_2.setValue(self._parent.currentID)
        self._ui.sbDaily_2.setValue(self._currentDailyNr)
        self._ui.pbBegin_2.clicked.connect(self._goBegin)
        self._ui.pbEnd_2.clicked.connect(self._goEnd)
        self._ui.pbBack_2.clicked.connect(self._goBack)
        self._ui.pbForward_2.clicked.connect(self._goForward)
        self._ui.twPlots.currentChanged.connect(self.updateTabPlots)

    def updateTabPlots(self):
        self._ui.sbID_2.setValue(self._parent.currentID)
        if self._ui.twMain.currentIndex() == self._parent.TWMAIN_PLOTS:
            if self._ui.twPlots.currentIndex() == self.PLTW_POWER:
                self.showPowerPlots()

    def getCoverage(self):
        (self._parent.fromId, self._parent.toId) = self._parent.dataSource.idCoverage(self._parent.fromDate,
                                                                                      self._parent.toDate)

        self._toDailyNr = self._parent.dataSource.dailyCoverage(self._parent.fromDate)
        self._currentDailyNr = 1
        self._parent.currentID = self._parent.fromId
        self._ui.sbID_2.setMinimum(self._parent.fromId)
        self._ui.sbID_2.setMaximum(self._parent.toId)
        self._ui.sbID_2.setValue(self._parent.currentID)
        self._ui.sbDaily_2.setMinimum(1)
        self._ui.sbDaily_2.setMaximum(self._toDailyNr)
        self._ui.sbDaily_2.setValue(self._currentDailyNr)

    def showPowerPlots(self):
        self._parent.busy(True)
        name, data, self._dailyNr, self._utcDate = self._parent.dataSource.extractDumpData(self._parent.currentID)
        print("_showPlots({}) {}".format(self._parent.currentID, name))

        layout = self._ui.wPowerContainer.layout()
        if layout is None:
            layout = QHBoxLayout()
        else:
            layout.removeWidget(self._plot)

        self._ui.sbDaily_2.setValue(self._dailyNr)
        scroller = QScrollArea()
        if name is not None:

            self._ui.lbDumpFilename.setText(name)



            #TBD...
        else:
            warning = QLabel()
            warning.setText("Nothing to show. The current event ID has no dump file associated.")
            warning.setStyleSheet("color:rgb(0, 255, 0); font: 14pt \"Gauge\"")
            scroller.setWidget(warning)

        layout.addWidget(scroller)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        self._ui.wPowerContainer.setLayout(layout)
        self._plot = scroller

        self._parent.busy(False)

    def _goBegin(self):
        """
        browsing screenshots and plots
        @return:
        """
        self._parent.currentID = self._parent.fromId
        self.updateTabPlots()

    def _goEnd(self):
        """
        browsing screenshots and plots
        @return:
        """
        self._parent.currentID = self._parent.toId
        self.updateTabPlots()

    def _goBack(self):
        """
        browsing screenshots and plots
        @return:
        """
        if self._parent.currentID > self._parent.fromId:
            self._parent.currentID -= 1
        self.updateTabPlots()

    def _goForward(self):
        """
        browsing screenshots and plots
        @return:
        """
        if self._parent.currentID < self._parent.toId:
            self._parent.currentID += 1
        self.updateTabPlots()



"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""

import os
import time
from platform import uname
from datetime import datetime
from pathlib import Path

from pyqtspinner import WaitingSpinner
from PyQt5.QtCore import Qt, QDate, QEvent, QTimer
from PyQt5.QtGui import QColor, QFontDatabase
from PyQt5.QtWidgets import QMainWindow, QApplication, QMessageBox

# from splash import SplashWindow
from .ui_mainwindow import Ui_ebrowWindow
from .settings import Settings
from .datasource import DataSource
from .screenshots import ScreenShots
from .rtsconfig import RTSconfig
from .stats import Stats
from .prefs import Prefs
from .utilities import createCustomColormapsDict, cleanFolder
from .reports import Report
from .logprint import print, logPrintVerbose

APPVERSION = "0.1.70"


class MainWindow(QMainWindow):
    TWMAIN_DB = 0
    TWMAIN_EVENTS = 1
    TWMAIN_STATISTICS = 2
    TWMAIN_RTS = 3
    TWMAIN_REPORT = 4
    TWMAIN_PREFS = 5
    WINDOW_TITLE = "Echoes Data Browser"

    def __init__(self, app: QApplication, batchRMOB: bool = False, batchReport: bool = False, verboseLog: bool = True):
        super(MainWindow, self).__init__()

        self.app = app
        self.version = APPVERSION
        self.toDate = None
        self.fromDate = None
        self.dataSource = None
        self.filteredIDs = []
        self.filteredDailies = []
        self.currentDate = None
        self.covDays = None
        self.dbOk = False

        self.covID = 0
        self.currentIndex = -1  # index in filteredIDs to get the currentID
        self.busyCount = 0
        self.currentID = 0
        self.currentDailyNr = 0
        self.toDailyNr = 0
        self.fromId = 0
        self.toId = 0
        self.coverage = 0
        self.quitting = False
        logPrintVerbose(verboseLog)

        self.cmapDict = createCustomColormapsDict()
        self.batchRMOB = batchRMOB
        self.batchReport = batchReport

        self._fontDB = QFontDatabase()
        self._fontDB.addApplicationFont(":/resources/fonts/Gauge-Regular.ttf");
        self._fontDB.addApplicationFont(":/resources/fonts/Gauge-Heavy.ttf");
        self._ui = Ui_ebrowWindow()
        self._ui.setupUi(self)
        self._splash = None
        self._tabScreenshots = None
        self.tabStats = None
        self.tabRTS = None
        self.tabReport = None
        self.tabPrefs = None
        self._cleanExportReq = False

        # list of events which class will be updated by pressing the DB update button
        self.classChanges = list()
        self.classifications = None
        self._ui.pbSave.setEnabled(False)
        self._ui.pbSubset.setEnabled(False)
        self._ui.pbClassReset.setEnabled(False)
        self._ui.pbDB.setVisible(False)

        self.appPath = os.path.dirname(os.path.realpath(__file__))
        print("appPath=", self.appPath)
        if not os.path.isdir(self.appPath):
            print("invalid appPath")
            if uname().system == 'Windows':
                # this is the exe version of ebrow
                # so, its appPath matches with Echoes one
                self.appPath = Path(os.environ['programfiles']) / "GABB" / "echoes"
                print("new appPath=", self.appPath)
            else:
                print("Unable to generate report, assets directory missing")
                return

        self._spinner = WaitingSpinner(
            self,
            roundness=52,
            fade=12,
            radius=90,
            lines=60,
            line_length=10,
            line_width=4,
            speed=0.3,
            color=QColor("powderblue")
        )

        self.filterTags = ['OVER', 'UNDER', 'FAKE ESD', 'FAKE CAR1', 'FAKE CAR2', 'FAKE SAT', 'FAKE LONG',
                           'ACQ ACT']
        self.filterChecks = [self._ui.chkOverdense, self._ui.chkUnderdense, self._ui.chkFakeEsd, self._ui.chkFakeCar1,
                             self._ui.chkFakeCar2, self._ui.chkFakeSat, self._ui.chkFakeLong, self._ui.chkFakeLong]
                             # last one is unused but these 2 lists must have same length

        self.filterCheckStats = [self._ui.chkOverdense_2, self._ui.chkUnderdense_2, self._ui.chkFakeEsd_2,
                                 self._ui.chkFakeCar1_2, self._ui.chkFakeCar2_2, self._ui.chkFakeSat_2,
                                 self._ui.chkFakeLong_2, self._ui.chkAcqActive_2]

        self.filterCheckReport = [self._ui.chkOverdense_3, self._ui.chkUnderdense_3, self._ui.chkFakeEsd_3,
                                  self._ui.chkFakeCar1_3, self._ui.chkFakeCar2_3, self._ui.chkFakeSat_3,
                                  self._ui.chkFakeLong_3]

        self._ui.lbVersion.setText(self.version)
        self._ui.gbSrvExport.setVisible(False)
        self.workingDir = Path(Path.home(), Path("ebrow"))
        self.exportDir = Path(self.workingDir, "exports")
        self._iniFilePath = str(Path(self.workingDir, "ebrow.ini"))
        self._settings = Settings(self._iniFilePath)
        self._settings.load()
        windowGeometry = self._settings.readSettingAsObject('geometry')
        self.setGeometry(windowGeometry)
        (self.fromDate, self.toDate) = self._settings.coverage()
        self.fromQDate = QDate().fromString(self.fromDate, 'yyyy-MM-dd')
        self.toQDate = QDate().fromString(self.toDate, 'yyyy-MM-dd')
        self._ui.dtFrom.setDate(self.fromQDate)
        self._ui.dtTo.setDate(self.toQDate)
        Path.mkdir(self.workingDir, parents=True, exist_ok=True)
        self.updateStatusBar("working directory {} ok ".format(self.workingDir))
        Path.mkdir(self.exportDir, parents=True, exist_ok=True)

        os.chdir(self.workingDir)
        self._ui.pbOpen.clicked.connect(self._openDataSource)
        self._ui.pbReload.clicked.connect(self._reloadDataSource)
        self._ui.pbSave.clicked.connect(self._updateDataSource)
        self._ui.pbSubset.clicked.connect(self._createSubset)
        self._ui.pbClassReset.clicked.connect(self._resetClassifications)
        self._ui.pbQuit.clicked.connect(self._quit)
        self._ui.twMain.currentChanged.connect(self._handleTabChanges)

        # the following tabs remain hidden until a db is opened:
        self._ui.twMain.setTabVisible(1, False)  # Screenshots
        self._ui.twMain.setTabVisible(2, False)  # Statistics
        self._ui.twMain.setTabVisible(3, False)  # RTS config
        self._ui.twMain.setTabVisible(4, False)  # Report
        self._initTimer = QTimer()
        self._initTimer.timeout.connect(self.postInit)
        self._initTimer.start(500)
        if not (batchRMOB or batchReport):
            # self._splash = SplashWindow(self, ":/splashscreen")
            # self._splash.show()
            pass

    # public methods:

    def checkExportDir(self, checkThisDir):
        if not self._cleanExportReq:
            self._cleanExportReq = True
            if not os.listdir(checkThisDir):
                self.updateStatusBar("export directory {} ok".format(checkThisDir))
            else:
                self.updateStatusBar("export directory {} not empty".format(checkThisDir))
                if not (self.batchRMOB or self.batchReport):
                    result = QMessageBox.question(self, self.WINDOW_TITLE,
                                                  "Export directory not empty, delete content before usage?")
                    if result == QMessageBox.StandardButton.Yes:
                        if cleanFolder(str(checkThisDir)):
                            self.updateStatusBar("export directory {} cleaned".format(checkThisDir))

    def eventFilter(self, obj, event):
        if self._settings.readSettingAsBool('tooltipDisabled'):
            if event.type() == QEvent.ToolTip:
                return True

        return super(MainWindow, self).eventFilter(obj, event)

    def postInit(self):
        """
        initialization tasks executed after the main window has been shown
        @return:
        """
        print("post-initialization")
        self._initTimer.stop()

        # the following tabs remain hidden until a db is opened:
        self._ui.twMain.setTabVisible(1, False)  # Screenshots
        self._ui.twMain.setTabVisible(2, False)  # Statistics
        self._ui.twMain.setTabVisible(3, False)  # RTS config
        self._ui.twMain.setTabVisible(4, False)  # Report
        self.updateStatusBar("Creating tabs")
        self._tabScreenshots = ScreenShots(self, self._ui, self._settings)
        self.tabStats = Stats(self, self._ui, self._settings)
        self.tabRTS = RTSconfig(self, self._ui, self._settings)
        self.tabPrefs = Prefs(self, self._ui, self._settings)
        self.tabReport = Report(self, self._ui, self._settings)

        self._ui.lbVersion.setText(APPVERSION)
        self.currentDate = self._settings.readSettingAsString('currentDate')

        # updates the selected range
        # to cover the current month until today
        today = datetime.today()
        if self.batchRMOB:
            self._ui.dtFrom.setDate(QDate(today.year, today.month, 1))
        else:
            dateFrom = self._settings.readSettingAsString('dateFrom')
            qdateFrom = QDate().fromString(dateFrom, 'yyyy-MM-dd')
            self._ui.dtFrom.setDate(qdateFrom)

        self._ui.dtTo.setDate(QDate(today.year, today.month, today.day))

        if self.batchRMOB or self.batchReport:
            self.updateStatusBar("Loading database file")
            self._openDataSource(noGUI=True)
            if self._ui.chkAutosave.isChecked() is True:
                self.updateStatusBar("Self saving the database file after classifications")
                self._updateDataSource()

            if self.batchRMOB:
                self.updateStatusBar("Self updating and sending RMOB files (--rmob specified)")
                self.getRMOBdata()
                self.close()

            elif self.batchReport:
                self.updateStatusBar("Self generating report (--report specified)")
                self.tabReport.updateTabReport()
                self.close()

        else:
            if self._splash:
                self._splash.end()
        # self._parent.checkExportDir(self._exportDir)
        self.busy(False, force=True)

    def getRMOBdata(self, sendOk: bool = True):
        # automatic export of RMOB file
        # for runtime and reporting
        return self.tabStats.updateRMOBfiles(sendOk)

    def getSummaryPlot(self, filters):
        # automatic creation of summary plot
        # for reporting
        return self.tabStats.updateSummaryPlot(filters)

    def closeEvent(self, event):
        # closing the window with the X button
        self._quit()
        event.accept()

    def resizeEvent(self, event):
        QMainWindow.resizeEvent(self, event)
        '''
        if self._tabScreenshots is not None:
            self._tabScreenshots.updateThumbnails()
        '''

    def busy(self, wantBusy: bool, force: bool = False, spinner: bool = True):
        if wantBusy:
            if self.busyCount == 0:
                # disable all pushbuttons when waiting
                self.updateStatusBar("Busy")
                self._ui.pbShotExp.setEnabled(False)
                self._ui.pbStatTabExp.setEnabled(False)
                self._ui.pbCfgTabExp.setEnabled(False)
                self._ui.pbRMOB.setEnabled(False)
                self._ui.pbRefresh.setEnabled(False)
                self._ui.pbRefresh_2.setEnabled(False)
                self._ui.pbRefresh_3.setEnabled(False)
                self._ui.pbReportXLSX.setEnabled(False)
                self._ui.pbReportHTML.setEnabled(False)
                self._ui.pbReset.setEnabled(False)
                self._ui.pbSave.setEnabled(False)
                self._ui.pbSubset.setEnabled(False)
                self._ui.pbReload.setEnabled(False)
                self._ui.pbOpen.setEnabled(False)
                self._ui.pbClassReset.setEnabled(False)
                self._ui.dtFrom.setEnabled(False)
                self._ui.dtTo.setEnabled(False)
                if spinner and self._spinner is not None:
                    self._spinner.start()
                    self._spinner.raise_()
                else:
                    QApplication.setOverrideCursor(Qt.WaitCursor)
            self.busyCount += 1
            print("Busy [{}]".format(self.busyCount))
        else:
            if self.busyCount > 0:
                self.busyCount -= 1
                if self.busyCount == 0 or force:
                    self.busyCount = 0
                    self.updateProgressBar(100) # hide progress bar
                    self.updateStatusBar("Ready")
                    # self.updateStatusBar("Ready (force={})".format(force))
                    self._ui.pbShotExp.setEnabled(True)
                    self._ui.pbStatTabExp.setEnabled(True)
                    self._ui.pbCfgTabExp.setEnabled(True)
                    self._ui.pbRMOB.setEnabled(True)
                    self._ui.pbRefresh.setEnabled(True)
                    self._ui.pbRefresh_2.setEnabled(True)
                    self._ui.pbRefresh_3.setEnabled(True)
                    self._ui.pbReportXLSX.setEnabled(True)
                    self._ui.pbReportHTML.setEnabled(True)
                    self._ui.pbReset.setEnabled(True)
                    self._ui.pbSave.setEnabled(True)
                    self._ui.pbSubset.setEnabled(True)
                    self._ui.pbReload.setEnabled(True)
                    self._ui.pbOpen.setEnabled(True)
                    self._ui.pbClassReset.setEnabled(True)
                    self._ui.dtFrom.setEnabled(True)
                    self._ui.dtTo.setEnabled(True)
                    if spinner and self._spinner is not None:
                        self._spinner.stop()
                    QApplication.restoreOverrideCursor()
                else:
                    pass
                    # print("Still busy [{}]".format(self.busyCount))
        self.app.processEvents()
        time.sleep(0.1)

    def updateStatusBar(self, text):
        print(text)
        s = "{} - {}".format(datetime.now(), text)
        self._ui.pteStatus.appendPlainText(s)
        self._ui.lbStatus.setText(text)
        if self.dataSource is not None:
            todayIDs = self.dataSource.getIDsOfTheDay(self.currentDate)
            if todayIDs is not None and todayIDs is not []:
                self._ui.lbTotal.setNum(self.covID)
                self._ui.lbDate.setText(self.currentDate)
                self._ui.lbCurDaily.setNum(self.toDailyNr)
                self._ui.lbCurFiltered.setNum(len(self.filteredIDs))
        if self.app is not None:
            self.app.processEvents()

    def updateProgressBar(self, percentage: int):
        if self.app is not None:
            self.app.processEvents()
        if percentage < 100:
            self._ui.pbDB.setValue(percentage)
            self._ui.pbDB.setVisible(True)
        else:
            self._ui.pbDB.setVisible(False)

    def infoMessage(self, notice: str, msg: str):
        self.busy(False, force=True)
        errorbox = QMessageBox()
        errorbox.setWindowTitle(self.WINDOW_TITLE)
        errorbox.setText(str(notice) + '\n' + str(msg))
        errorbox.setIcon(QMessageBox.Information)
        errorbox.exec_()

    def confirmMessage(self, notice: str, msg: str):
        self.busy(False, force=True)
        errorbox = QMessageBox()
        errorbox.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
        errorbox.setWindowTitle(self.WINDOW_TITLE)
        errorbox.setText(str(notice) + '\n' + str(msg))
        errorbox.setIcon(QMessageBox.Question)
        button = errorbox.exec_()
        return button == QMessageBox.Ok

    # protected methods:

    def _handleTabChanges(self):
        idx = self._ui.twMain.currentIndex()
        if idx == self.TWMAIN_EVENTS:
            self._tabScreenshots.updateTabEvents()
        if idx == self.TWMAIN_STATISTICS:
            self.tabStats.updateTabStats()
        if idx == self.TWMAIN_RTS:
            self.tabRTS.updateTabRTSconfig()
        if idx == self.TWMAIN_PREFS:
            self.tabPrefs.updateTabPrefs()
        if idx == self.TWMAIN_REPORT:
            self.tabReport.updateTabReport()

    def _getCoverage(self):
        if self.dataSource is None:
            return

        qDateFrom = self._ui.dtFrom.date()
        qDateTo = self._ui.dtTo.date()
        defaultDate = self._settings.readSettingAsString('currentDate')
        defaultQDate = QDate().fromString(defaultDate, 'yyyy-MM-dd')

        self.fromDate = qDateFrom.toString("yyyy-MM-dd")
        self.toDate = qDateTo.toString("yyyy-MM-dd")
        self.coverage = qDateFrom.daysTo(qDateTo) + 1
        self.covDays = list()
        date = qDateFrom
        for day in range(0, self.coverage):
            dateStr = date.toString("yyyy-MM-dd")
            self.filteredIDs, self.filteredDailies = self.dataSource.getFilteredIDsOfTheDay(dateStr)
            if len(self.filteredIDs) > 0:
                self.covDays.append(dateStr)
                if defaultDate == '' or not qDateFrom <= defaultQDate <= qDateTo:
                    # if default date is invalid
                    self.currentDate = dateStr  # takes by default the most recent date with images
            date = date.addDays(1)

        self.updateStatusBar("coverage {} days, from {} to {}:".format(self.coverage, self.fromDate, self.toDate))
        self.filteredIDs, self.filteredDailies = self.dataSource.getFilteredIDsOfTheDay(self.currentDate)
        self._ui.lbCovFrom.setText(self.fromDate)
        self._ui.lbCovTo.setText(self.toDate)
        self._settings.writeSetting('dateFrom', self.fromDate)
        self._settings.writeSetting('dateTo', self.toDate)
        if self._tabScreenshots.getCoverage() is False:
            self._tabScreenshots.getCoverage(selfAdjust=True)
            dt = time.strptime(self.fromDate, "%Y-%m-%d")
            self._ui.dtFrom.setDate(QDate(dt.tm_year, dt.tm_mon, dt.tm_mday))
            dt = time.strptime(self.toDate, "%Y-%m-%d")
            self._ui.dtTo.setDate(QDate(dt.tm_year, dt.tm_mon, dt.tm_mday))
            # FIXME: recursion max 1 loop
            self._getCoverage()
            self.updateStatusBar("Fixed day coverage from {} to {}, total {} IDs, from {} to {}:".format(
                self.fromDate, self.toDate, self.covID, self.fromId, self.toId))
            return

        self.covID = (self.toId - self.fromId) + 1
        self.updateStatusBar("coverage {} IDs, from {} to {}:".format(self.covID, self.fromId, self.toId))

    def _quit(self):
        if not self.quitting:
            self.quitting = True
            if self.dataSource is not None:
                if not (self.batchRMOB or self.batchReport):
                    self._updateDataSource()
                self.dataSource.closeFile()
            self._settings.writeSetting('geometry', self.geometry())
            qDateFrom = self._ui.dtFrom.date()
            qDateTo = self._ui.dtTo.date()
            self.fromDate = qDateFrom.toString("yyyy-MM-dd")
            self.toDate = qDateTo.toString("yyyy-MM-dd")
            self._settings.writeSetting('dateFrom', self.fromDate)
            self._settings.writeSetting('dateTo', self.toDate)
            self._settings.save()
            if not (self.batchRMOB or self.batchReport):
                self.close()

    def _updateDataSource(self):
        print("_updateDataSource()")
        if self.dataSource is not None:
            if True in self.classChanges:
                if (self._settings.readSettingAsBool('autosaving')
                        or
                        self.confirmMessage("Warning - updated classifications",
                                            "Press OK to update the database file on disk, Cancel to skip")):
                    self.busy(True)
                    if self._overrideClassifications():
                        self.updateStatusBar("Saving classifications in DB...")
                        if self.dataSource.updateFile():
                            self.updateStatusBar("Classifications saved in DB")
                            self.classChanges = [False] * (self.fromId + self.covID)
                            self.busy(False)
                            return True
                    self.busy(False)

            elif self.dataSource.dataChanged():
                if (self._settings.readSettingAsBool('autosaving')
                        or self.confirmMessage("Warning - cleared classifications",
                                               "Press OK to update the database file on disk, Cancel to skip")):

                    self.updateStatusBar("Clearing classifications in DB...")
                    self.busy(True)
                    if self.dataSource.updateFile():
                        self.updateStatusBar("Classifications reset in DB")
                        self.classChanges = [False] * (self.fromId + self.covID)
                        self.busy(False)
                        return True
                    self.busy(False)
        print("No changes to be saved in DB")
        return False

    def _createSubset(self):
        """

        """
        print("_createSubset()")
        if self.dataSource is not None:
            # open save dialog
            subsetDBpath = self.dataSource.getSubsetFilename()
            if len(subsetDBpath) > 0:
                self.busy(True)
                self.updateStatusBar("Creating a DB subset into {}".format(subsetDBpath))
                result = self.dataSource.createSubset(subsetDBpath, self.fromDate, self.toDate)

                self.busy(False)

    def _reloadDataSource(self):
        """

        """
        self._openDataSource(noGUI=True)

    def _resetClassifications(self):
        if self.confirmMessage("Warning",
                               "Press OK to reset all classifications, Cancel to skip.\n To clear the classifications"
                               " on disk too, press Update Data Source button afterwards."):
            self.busy(True)
            self.updateStatusBar("Clearing all classifications in memory")
            self.dataSource.resetClassifications()
            self.updateStatusBar("Done. Press Update Data Source to reset classifications on DB file also.")
            self.busy(False)

    def _overrideClassifications(self):
        """
        updates the classifications in automatic_data dataframe
        with manual overrides, if any
        :return:
        """
        self.busy(True)
        self.updateStatusBar("Updating classifications on DB...")
        idx = 0
        updated = 0
        progressPercent = 0
        total = len(self.classChanges)
        self.updateProgressBar(progressPercent)
        for xchg in self.classChanges:
            if xchg is True:
                self.dataSource.setEventClassification(idx, self.classifications.loc[idx, 'classification'])
            idx += 1
            updated += 1
            r = idx % 5
            progress = (idx / total) * 100
            if progress % 5 < r:
                progressPercent += 5
                self.updateProgressBar(progressPercent)

        self.updateStatusBar("{} classifications ready, including manual overrides".format(updated - 1))
        # self._ui.pbSave.setEnabled(False)
        # self._ui.pbClassReset.setEnabled(False)
        self.busy(False)
        return True

    def _openDataSource(self, noGUI: bool = False):
        """

        @param noGUI:
        @return:
        """


        try:
            self._ui.dtFrom.dateChanged.disconnect()
            self._ui.dtTo.dateChanged.disconnect()
        except TypeError:
            pass

        self.dataSource = DataSource(self, self._settings)
        self.dbOk = False
        name = self._settings.readSettingAsString('lastDBfilePath')

        if name != 'None' and noGUI:
            # automatic database opening at startup
            dbFile = self.dataSource.openFile(name)
            if dbFile is None:
                self.updateStatusBar("Failed opening stored database file {}".format(name))
            else:
                self.dbOk = True

        if not noGUI:
            # database opening required by the user via GUI
            self._ui.lbDBfilename.setText("loading...")
            if self.dataSource.openFileDialog() is None:
                self.updateStatusBar("Failed opening new database file")

            else:
                self.dbOk = True

        (qDateFrom, qDateTo) = self.dataSource.QDateCoverage()
        if self.dbOk and (qDateFrom is None or qDateTo is None):
            self.dbOk = False

        if self.dbOk:
            self.busy(True)
            # self._ui.dtFrom.blockSignals(True)
            # self._ui.dtTo.blockSignals(True)
            self._ui.lbDBfilename.setText("{} ".format(self.dataSource.name()))
            if not (self.fromQDate >= qDateFrom and self.toQDate <= qDateTo):
                # if the current interval is not included in data source coverage
                # the interval is reinitialized from data source
                self._ui.dtFrom.setDate(qDateFrom)
                self._ui.dtTo.setDate(qDateTo)

            self._getCoverage()
            self._ui.twMain.setTabVisible(1, True)  # Screenshots
            self._ui.twMain.setTabVisible(2, True)  # Plots
            self._ui.twMain.setTabVisible(3, True)  # Statistics
            self._ui.twMain.setTabVisible(4, True)  # Report
            self.tabPrefs.updateTabPrefs()
            self._ui.dtFrom.dateChanged.connect(self._getCoverage)
            self._ui.dtTo.dateChanged.connect(self._getCoverage)

            if len(self.filteredIDs) > 0:
                self.currentID = self.filteredIDs[self.currentIndex]
                self.currentDailyNr = self.filteredDailies[self.currentIndex]
            self.updateStatusBar("DB: {}".format(self.dataSource.fullPath()))
            self._settings.writeSetting('lastDBfilePath', self.dataSource.fullPath())
            self.updateStatusBar("Opening ok, performing classifications...")
            self.classChanges = [False] * (self.fromId + self.covID)
            self.dataSource.classifyEvents(self.fromId, self.toId)
            self._ui.pbSave.setEnabled(True)
            self._ui.pbSubset.setEnabled(True)
            self._ui.pbClassReset.setEnabled(True)
            # initialize the editable classifications list

            self.classifications = self.dataSource.getEventClassifications(self.fromId, self.toId)
            self.updateStatusBar("Classifications loaded")
            self.busy(False)

        else:
            # the following tabs remain hidden until a db is opened:
            self._ui.twMain.setTabVisible(1, False)  # Screenshots
            self._ui.twMain.setTabVisible(2, False)  # Statistics
            self._ui.twMain.setTabVisible(3, False)  # RTS config



"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""
from datetime import datetime, timezone
from dateutil.rrule import SECONDLY, MINUTELY

import pandas as pd
import numpy as np
import matplotlib as mp
import matplotlib.pyplot as plt
from matplotlib.dates import AutoDateLocator, date2num, MICROSECONDLY, DateFormatter
from mplcursors import cursor
from matplotlib.ticker import MaxNLocator, ScalarFormatter
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg

from .utilities import PrecisionDateFormatter
from .settings import Settings
from .basegraph import BaseGraph
from .logprint import print
mp.use('Qt5Agg')


class MapPlot(BaseGraph):
    def __init__(self, dfMap: pd.DataFrame, dfPower: pd.DataFrame, settings: Settings, inchWidth: float,
                 inchHeight: float, cmap: list,
                 name: str, vmin: float, vmax: float, tickEveryHz: int = 1000, tickEverySecs: int = 1,
                 showGrid: bool = True):
        BaseGraph.__init__(self, settings)

        plt.rcParams['axes.autolimit_mode'] = 'round_numbers'

        # dfMap.to_csv('C:/temp/map.csv', sep=';')
        dfMap = dfMap.reset_index()

        # --- horizontal x axis [Hz] ----

        # FFT bins
        freqs = dfMap['frequency'].unique()
        totFreqs = len(freqs)
        xLims = [freqs[0], freqs[-1]]
        freqSpan = dfMap['frequency'].max() - dfMap['frequency'].min()

        nTicks = (freqSpan / tickEveryHz) - 1
        xLoc = MaxNLocator(nTicks, steps=[1, 2, 5], min_n_ticks=nTicks)
        xFmt = ScalarFormatter()

        # --- vertical Y axis [sec] ----

        # data scans
        scans = dfPower.index.unique().to_list()
        totScans = len(scans)
        dt = datetime.fromtimestamp(scans[0], tz=timezone.utc)
        startTime = np.datetime64(dt)
        dt = datetime.fromtimestamp(scans[-1], tz=timezone.utc)
        endTime = np.datetime64(dt)

        yLims = date2num([startTime, endTime])

        yLoc = AutoDateLocator(interval_multiples=True)
        if tickEverySecs > 120.0:
            tickEveryMins = tickEverySecs / 60
            yLoc.intervald[MINUTELY] = [tickEveryMins]
        elif tickEverySecs < 1.0:
            tickEveryUs = tickEverySecs * 1E6
            # yLoc.intervald[7] = [tickEveryUs]
            yLoc.intervald[MICROSECONDLY] = [tickEveryUs]
        else:
            yLoc.intervald[SECONDLY] = [tickEverySecs]

        # note: MICROSECONDLY needs matplotlib 3.6.0++ and Python 3.8++

        # yFmt = PrecisionDateFormatter('%H:%M:%S.%f', tz=timezone(timedelta(0)))
        yFmt = DateFormatter('%H:%M:%S.%f')

        # ---- the waterfall flows downwards so the time increase from bottom to top (origin lower)

        data = dfMap[['S']].to_numpy().reshape(totScans, totFreqs)
        self._min = data.min()
        self._max = data.max()

        np.clip(data, vmin, vmax, data)

        colors = self._settings.readSettingAsObject('colorDict')
        backColor = colors['background'].name()
        plt.figure(figsize=(inchWidth, inchHeight))
        self._fig, ax = plt.subplots(1, facecolor=backColor)
        ax.set_facecolor(backColor)

        im = ax.imshow(data, cmap=cmap, aspect='auto', vmin=vmin, vmax=vmax, interpolation=None,
                       origin='lower', extent=[xLims[0], xLims[1], yLims[0], yLims[1]])
        print("extent=", im.get_extent())

        ax.xaxis.set_major_locator(xLoc)
        ax.xaxis.set_major_formatter(xFmt)
        ax.set_xlabel('frequency [Hz]', labelpad=30)

        ax.yaxis.set_major_locator(yLoc)
        ax.yaxis.set_major_formatter(yFmt)
        ax.set_ylabel('time of day', labelpad=30)

        norm = mp.colors.Normalize(vmin=vmin, vmax=vmax)
        self._fig.colorbar(im, drawedges=False, norm=norm, cmap=cmap)
        title = "Mapped spectrogram from data file " + name
        ax.set_title(title, loc='left', pad=20)
        ax.tick_params(axis='x', which='both', labelrotation=90, color=colors['majorGrids'].name())

        if showGrid:
            ax.grid(which='major', axis='both', linestyle=self._settings.readSettingAsString('majorLineStyle'),
                    linewidth=self._settings.readSettingAsString('majorLineWidth'), color=colors['majorGrids'].name())

        if self._settings.readSettingAsString('cursorEnabled') == 'true':
            cursor(hover=True)

        self._df = dfMap
        self._fig.set_tight_layout({"pad": 5.0})
        self._canvas = FigureCanvasQTAgg(self._fig)
        # avoids showing the original fig window
        plt.close('all')

    def savePlotDataToDisk(self, fileName):
        self._df = self._df.set_index('time')
        self._df.to_csv(fileName, sep=self._settings.dataSeparator())

    def zoom(self, horz: int, vert: int):
        self._fig.set_size_inches(horz, vert)
        self._canvas.draw()
        # plt.pause(0.01)

    def getMinMax(self):
        return [self._min, self._max]



"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""

import matplotlib as mp
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg

from .settings import Settings
from .basegraph import BaseGraph
from .logprint import print
mp.use('Qt5Agg')


class NestedPie(BaseGraph):
    def __init__(self, df: pd.DataFrame, settings: Settings, inchWidth: float, inchHeight: float, subtractBackground: bool=False):
        BaseGraph.__init__(self, settings)

        df['sum'] = df.sum(axis='columns')
        totals = df.loc[0, 'sum']
        totalOver = df.loc[0, 'OVER']
        totalUnder = df.loc[0, 'UNDER']
        totalFakes = totals - totalOver - totalUnder
        cats = list()
        subcats = list()
        valuesCats = list()
        valuesSubcats = list()
        colors = self._settings.readSettingAsObject('colorDict')
        if totalUnder > 0:
            count = totalUnder
            percentage = np.round((count / totals) * 100.0, 2)
            valuesCats.append(count)
            cats.append("Underdense:\n{} = {}%".format(count, percentage))
            subcats.append(' ')
            valuesSubcats.append(count)

        if totalOver > 0:
            count = totalOver
            percentage = np.round((count / totals) * 100.0, 2)
            valuesCats.append(count)
            cats.append("Overdense:\n{} = {}%".format(count, percentage))
            subcats.append(' ')
            valuesSubcats.append(count)

        if totalFakes > 0:
            count = totalFakes
            percentage = np.round((count / totals) * 100.0, 2)
            valuesCats.append(count)
            cats.append("Fakes:\n{} = {}%".format(count, percentage))

            count = df.loc[0, 'FAKE_CAR1']
            if count > 0:
                percentage = np.round((count / totals) * 100.0, 2)
                subcats.append("carriers 1:\n{} = {}%".format(count, percentage))
                valuesSubcats.append(count)
            count = df.loc[0, 'FAKE_CAR2']
            if count > 0:
                percentage = np.round((count / totals) * 100.0, 2)
                subcats.append("carriers 2:\n{} = {}%".format(count, percentage))
                valuesSubcats.append(count)
            count = df.loc[0, 'FAKE_ESD']
            if count > 0:
                percentage = np.round((count / totals) * 100.0, 2)
                subcats.append("ESD:\n{} = {}%".format(count, percentage))
                valuesSubcats.append(count)
            count = df.loc[0, 'FAKE_SAT']
            if count > 0:
                percentage = np.round((count / totals) * 100.0, 2)
                subcats.append("Saturations:\n{} = {}%".format(count, percentage))
                valuesSubcats.append(count)
            count = df.loc[0, 'FAKE_LONG']
            if count > 0:
                percentage = np.round((count / totals) * 100.0, 2)
                subcats.append("Too long:\n{} = {}%".format(count, percentage))
                valuesSubcats.append(count)

        cmap = plt.colormaps['tab20c']
        colorsCats = cmap([0, 4, 16])
        colorsSubCats = cmap([1, 5, 17, 14, 18, 19, 15])  # 1=under, 5=over, 16++ fakes...

        backColor = colors['background'].name()
        plt.figure(figsize=(inchWidth, inchHeight))
        self._fig, ax = plt.subplots(1, facecolor=backColor)
        ax.set_facecolor(backColor)

        plt.pie(valuesCats, labels=cats, colors=colorsCats, startangle=180, frame=True)
        plt.pie(valuesSubcats, labels=subcats, colors=colorsSubCats, radius=0.7, startangle=180,
                labeldistance=0.7)
        center = plt.Circle((0, 0), 0.4, colors['majorGrids'].name(), linewidth=0)
        ax.annotate("total:\n\n {} = 100%".format(totals), xy=(0, 0), ha="center")
        title = "total events by classification"
        if subtractBackground:
            title += "\nafter sporadic background subtraction"
        ax.set_title(title)
        fig = plt.gcf()
        fig.gca().add_artist(center)
        plt.axis('equal')
        self._fig.set_tight_layout({"pad": 5.0})
        self._canvas = FigureCanvasQTAgg(self._fig)

    def zoom(self, horz: int, vert: int):
        self._fig.set_size_inches(horz, vert)
        self._canvas.draw()


# This Python file uses the following encoding: utf-8
from __future__ import print_function
import builtins as __builtin__
from datetime import datetime

verbose = False


def logPrintVerbose(wantVerbose: bool = False):
    global verbose
    verbose = wantVerbose


def print(*args, **kwargs):
    """
    allows to inhibit console prints() when
    --verbose switch is not specified
    :param args:
    :param kwargs:
    :return:
    """
    if verbose:
        utc = datetime.utcnow()
        __builtin__.print(utc, end=' - ')
        __builtin__.print(*args, **kwargs)

def fprint(*args, **kwargs):
    """
    allows prints on files, bypassing
    the --verbose switch
    
    :param args: 
    :param kwargs: 
    :return: 
    """
    """
    :param args: 
    :param kwargs: 
    :return: 
    """
    __builtin__.print(*args, **kwargs)


"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""
import io
import os
import sys
import traceback
import ctypes
from datetime import datetime
from PyQt5.QtWidgets import QApplication, QMessageBox
from PyQt5.QtCore import QSharedMemory

from .mainwindow import MainWindow

app = None
mainWin = None


def excepthook(excType, excValue, tracebackobj):
    global mainWin

    """
    Global function to catch unhandled exceptions
    @param excType exception type
    @param excValue exception value
    @param tracebackobj traceback object
    """

    mainWin.busy(False, force=True)
    separator = '-' * 80
    logFile = "ebrow.log"
    notice = \
        """An unhandled exception occurred. Please report the problem\n""" \
        """using the error reporting dialog or via email to <%s>.\n""" \
        """A log has been written to "%s".\n\nError information:\n""" % \
        ("gm_bertani@yahoo.it", logFile)
    versionInfo = "GUI version: " + mainWin.version
    timeString = str(datetime.today())

    tbinfofile = io.StringIO()
    traceback.print_tb(tracebackobj, None, tbinfofile)
    tbinfofile.seek(0)
    tbinfo = tbinfofile.read()
    errmsg = 'Exception %s: \n%s' % (str(excType), str(excValue))
    sections = [separator, timeString, separator, errmsg, separator, tbinfo, separator, versionInfo]
    msg = '\n'.join(sections)
    print(msg)
    try:
        f = open(logFile, "w")
        f.write(msg)
        f.write(versionInfo)
        f.close()
    except IOError:
        pass
    errorbox = QMessageBox()
    errorbox.setWindowTitle("Echoes Data Browser")
    errorbox.setText(str(notice) + str(msg))
    errorbox.setIcon(QMessageBox.Critical)
    errorbox.exec_()
    os.abort()


def main():
    global mainWin

    # sys.stdout = None  # suppress console output
    sys.excepthook = excepthook
    app = QApplication(sys.argv)

    batchRMOB = False
    batchReport = False
    verboseLog = False
    multipleInstances = False
    argc = len(sys.argv)
    while argc > 1:
        argc -= 1
        if '--rmob' in sys.argv[argc]:
            batchRMOB = True
        if '--report' in sys.argv[argc]:
            batchReport = True
        if '--verbose' in sys.argv[argc]:
            verboseLog = True
        if '--multiple' in sys.argv[argc]:
            multipleInstances = True
        if '--help' in sys.argv[argc]:
            print("Usage: ebrow [--verbose] [--report] [--rmob] [--multiple]")
            sys.exit(0)

    shm = QSharedMemory()
    shm.setKey("EbrowInstanceKey")
    unique = shm.create(1)
    if not unique:
        if not multipleInstances:
            print("ERROR - Cannot run. Another Ebrow instance is already running or has crashed.")
            print("        Use ebrow --multiple to bypass this lock.")
            sys.exit(-1)

    mainWin = MainWindow(app, batchRMOB, batchReport, verboseLog)
    if os.name == 'nt':
        myappid = 'GABB.Echoes.DataBrowser'
        ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)

    app.installEventFilter(mainWin)
    mainWin.show()
    # mainWin.busy(True)

    if batchRMOB or batchReport:
        mainWin.postInit()
        # note: mandatory under Linux, otherwise the segment remains allocated.
        shm.detach()
    else:
        ret = app.exec_()
        # note: mandatory under Linux, otherwise the segment remains allocated.
        shm.detach()
        sys.exit(ret)


if __name__ == '__main__':
    main()



"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""

from PyQt5.QtWidgets import QColorDialog, QWidget, QButtonGroup, QFileDialog
from PyQt5.QtGui import QColor, QIcon, QPixmap
from datetime import datetime
from .logprint import print
from .dateintervaldialog import DateIntervalDialog
from .utilities import splitASCIIdumpFile, splitBinaryDumpFile, mkExportFolder


class HasHead(QWidget):
    """
    This filter detects the presence of a head echo.
    It cannot rely on Raise front data uniquely, they
    must be integrated with information taken
    from the related dump file, so this filter
    cannot work if dumps are disabled and evalFilter()
    will return always False in this case
    """
    def __init__(self, parent, ui, settings):
        self._ui = ui
        self._parent = parent
        self._settings = settings

    def evalFilter(self, evId: int) -> bool:
        """
        calculates if the given event satisfies this filter
        """
        df = self._parent.datasource.getEventData(evId)

        datName, datData, dailyNr, utcDate = self._parent.dataSource.extractDumpData(evId)
        if datName is not None and datData is not None:
            if ".datb" in datName:
                dfMap, dfPower = splitBinaryDumpFile(datData)
            else:
                dfMap, dfPower = splitASCIIdumpFile(datData)

    def getParameters(self):
        """
        displays the parametrization dialog
        and gets the user's settings
        """
        pass

    def load(self):
        """
        loads this filter's parameters
        from settings file
        """
        pass

    def save(self):
        """
        save ths filter's parameters
        to settings file
        """
        pass



"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""

from PyQt5.QtWidgets import QSplashScreen
from PyQt5.QtGui import QPixmap
from PyQt5.QtCore import Qt
from .logprint import print

class SplashWindow(QSplashScreen):
    def __init__(self, mainWindow, pixFilePath):
        pix = QPixmap(pixFilePath)
        print("pix=",pix)
        self._mainWindow = mainWindow
        super(SplashWindow, self).__init__(pix, Qt.WindowStaysOnTopHint)

    # public methods:
    def end(self):
        self.finish(self._mainWindow)


__all__ = [
"main.py",
"bargraph.py",
"basegraph.py",
"bg_rmob.py",
"datasource.py",
"ebrow_rc.py",
"heatmap.py",
"hm_rmob.py",
"mainwindow.py",
"mapplot.py",
"mapplot3d.py",
"nestedpie.py",
"pandasmodel.py",
"plots.py",
"powerplot.py",
"prefs.py",
"reports.py",
"rtsconfig.py",
"screenshots.py",
"settings.py",
"splash.py",
"statplot.py",
"stats.py",
"thumbnails_browser.py",
"ui_mainwindow.py",
"utilities.py,"
"logprint.py"
]



"""
******************************************************************************

    Echoes Data Browser (Ebrow) is a data navigation and report generation
    tool for Echoes.
    Echoes is a RF spectrograph for SDR devices designed for meteor scatter
    Both copyright (C) 2018-2023
    Giuseppe Massimo Bertani gm_bertani(a)yahoo.it

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, http://www.gnu.org/copyleft/gpl.html

*******************************************************************************

"""

import pandas as pd
import numpy as np
import matplotlib as mp
import matplotlib.pyplot as plt
from PyQt5.QtCore import QDate
from matplotlib.colors import ListedColormap
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg

from .settings import Settings
from .basegraph import BaseGraph
from .logprint import print

mp.use('Qt5Agg')


class HeatmapRMOB(BaseGraph):
    def __init__(self, dataFrame: pd.DataFrame, settings: Settings, inchWidth: float, inchHeight: float, cmap: ListedColormap):
        BaseGraph.__init__(self, settings, 'RMOB')

        colors = self._settings.readSettingAsObject('colorDict')

        if len(dataFrame.columns) == 24:
            # 24 columns, one for each hour count from 00h to 23h
            df = dataFrame
            days = len(df.index)
            # calculate full-scale value (red color) taking the maximum in all the counts columns
            countsOnlyDf = df.iloc[0:, 0:]
            self._fullScale = countsOnlyDf.max(axis=1).max()

            # rows header (hours)
            rowsHdr = list()
            # rowsHdr.append("Day / Hour")
            for hour in range(0, 24):
                s = "{:02d}h".format(hour)
                rowsHdr.append(s)

            # columns header (days)
            allColumns = list()
            colsHdr = list()
            for dayCount in range(0, days):
                isoDate = str(df.index[dayCount])
                if len(isoDate.split('-')) == 3:
                    date = QDate.fromString(isoDate, "yyyy-MM-dd")
                    s = date.day()
                else:
                    s = isoDate
                colsHdr.append(s)

                # days columns values
                column = list()
                for hour in range(0, 24):
                    value = int(df.iloc[dayCount, hour])
                    column.append(value)
                allColumns.append(column)
        else:
            return

        colsHdr = ['01', ' ', ' ', ' ', ' D', ' ', ' a', ' ', ' y', ' ', ' s', ' ', ' ', ' ', '15', ' ', ' ', ' ',
                   ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', '  ', '  ', '31']
        rowsHdr = ['00h', '   ', 'UT ', '   ', '   ', '   ',
                   '06h', '   ', '   ', '   ', '   ', '   ',
                   '12h', '   ', '   ', '   ', '   ', '   ',
                   '18h', '   ', '   ', '   ', '   ', '   ', ]

        data = np.array(allColumns)
        data = np.transpose(data)

        cmap.set_bad(color='black')
        data = data.astype(float)
        data[data == -1] = np.nan
        plt.figure(figsize=(inchWidth, inchHeight))
        self._fig, ax = plt.subplots(1, facecolor='#ffffff')
        ax.set_facecolor('#ffffff')

        im = ax.imshow(data, cmap=cmap, aspect='auto')

        # Show all ticks and label them with the respective list entries
        ax.spines[:].set_visible(False)
        xTicks = len(colsHdr)
        yTicks = len(rowsHdr)
        ax.set_xticks(np.arange(xTicks), labels=colsHdr)  # , minor=True)
        ax.set_yticks(np.arange(yTicks), labels=rowsHdr)  # , minor=True)
        # ax.grid(which="minor", color="w", linestyle='-', linewidth=2)
        # ax.tick_params(which="both", bottom=False, left=False, labelrotation=0)
        ax.tick_params(which="both", labelrotation=0, width=2)
        # ax.xaxis.set_ticks_position('bottom')
        ax.xaxis.set_ticks_position('top')
        ax.yaxis.set_ticks_position('left')
        # ax.xaxis.set_label_position('top')
        plt.setp(ax.get_xticklabels(), ha="center")
        plt.hlines(y=np.arange(0, yTicks) + 0.5, xmin=np.full(yTicks, 0) - 0.5,
                   xmax=np.full(yTicks, xTicks) - 0.5, color="white")
        plt.vlines(x=np.arange(0, xTicks) + 0.5, ymin=np.full(xTicks, 0) - 0.5,
                   ymax=np.full(xTicks, yTicks) - 0.5, color="white")

        valFmt = mp.ticker.StrMethodFormatter('{x:.0f}')
        norm = mp.colors.Normalize(vmin=0, vmax=int(self._fullScale))
        self._fig.colorbar(im, drawedges=False, norm=norm, cmap=cmap)
        self._fig.set_tight_layout({"pad": 0})
        self._canvas = FigureCanvasQTAgg(self._fig)
        # avoids showing the original fig window
        plt.close('all')

    def zoom(self, horz: int, vert: int):
        self._fig.set_size_inches(horz, vert)
        self._canvas.draw()
